# Premium packaging

The Trento premium packaging process is different than the community one.
The premium packages, containers and charts are shipped using the SUSE official channels.

This document aims to explain some concepts and processes to generate these deliverables.

Most of the information is based on the next confluence page: https://confluence.suse.com/pages/viewpage.action?spaceKey=ENGCTNRSTORY&title=Building+Container+Images+and+Helm+Charts#BuildingContainerImagesandHelmCharts-HelmCharts

The steps to release a new release mentioned in the next chapters are performed by the project CI/CD pipelines.

## Premium containers

The Trento Web and Runner components are shipped as containers in SUSE registries.
These are the used registries:

- Official registry: https://registry.suse.com
- Internal registry: https://registry.suse.de

In order to publish in this registries, the containers must be created in IBS.
The project containers are stored here: https://build.suse.de/project/show/Devel:SAP:trento:factory

- [trento-web-image](https://build.suse.de/package/show/Devel:SAP:trento:factory/trento-web-image): Trento web container image
- [trento-runner-image](https://build.suse.de/package/show/Devel:SAP:trento:factory/trento-runner-image): Trento runner container image
- [trento-db-image](https://build.suse.de/package/show/Devel:SAP:trento:factory/trento-db-image): Custom postgresql container image. It is a plain postgresql container, the selected name only aims to hide the implementation details, and attach this container to Trento

The content of these containers in IBS is simply copied from the subfolder in this project.

These steps must be done in order to release a new version:

- Update the `.changes` file must be updated manually adding a new entry describing the changes related to the container files
- Upgrade the image revision version. This version is defined in the `Dockerfile`. In the `revx.y.x` entry, in the second and third `BuildTag` comments. By now, this version is increased manually.
- Upload a new `Dockerfile` or auxiliary files

Once any of these steps is done, the container is automatically generated and published in `registry.suse.de`

## Premium chart

The Trento premium chart aims to provide a customized helm chart with different set of values. This way, the installed Trento environment uses the premium containers.

The project charts are stored here: https://build.suse.de/project/show/Devel:SAP:trento:factory

- [trento-server-helm](https://build.suse.de/package/show/Devel:SAP:trento:factory/trento-server-helm): Trento server helm chart

In order to create a new version follow the next steps:

- Update the `.changes` file. It must be updated manually adding a new entry describing the changes related to the chart files
- Update the needed values in `values-overwrite.yaml`. This entries overwrite the values in the community helm chart values file.
- Upgrade the chart revision version. This version is in the community helm chart `Chart.yaml` file. In the `x.y.z` entry,  in the first and second `BuildTag` comments. By now, this version is increased manually.

These changes belong to the chart update. In order to publish them in IBS follow the next steps:
**This is only available in networks with access to the R&D network!!**

- Checkout the IBS project in your machine. For that run: `osc -A https://api.suse.de checkout Devel:SAP:trento:factory/trento-server-helm  && cd $_`
- Update the downloaded files with the changes made in the project itself.
- Run the makefile: `make`. This step creates the tarball and additional files
- Upload the changes. For that: `osc ar` and `osc commit -m "New development version of trento-server-helm released"`

These steps updates the package in IBS, which publishes the helm chart in `registry.suse.de`.

In order to test the chart `helm` can be used. Follow the next steps:

- To test the official chart run:

```
helm upgrade --install trento-server oci://registry.suse.com/trento/trento-server:x.y.z --set-file trento-runner.privateKey=/root/.ssh/id_rsa
```

- To test the chart in the internal registry:

```
helm upgrade --install trento-server oci://registry.suse.de/devel/sap/trento/factory/charts/trento/trento-server:0.2.4 --set-file trento-runner.privateKey=/root/.ssh/id_rsa --set trento-web.image.repository=registry.suse.de/devel/sap/trento/factory/containers/trento/trento-web --set trento-runner.image.repository=registry.suse.de/devel/sap/trento/factory/containers/trento/trento-runner --set trento-web.image.tag=latest --set trento-runner.image.tag=latest -set postgresql.image.repository=devel/sap/trento/factory/containers/trento/trento-db --set postgresql.image.registry=registry.suse.de --set postgresql.image.tag=latest --set postgresql.image.pullPolicy=Always
```

If there is some sort of issue with the containers not being updated, uninstalling the currently running deployment and running again the installation might help. To uninstall:

```
helm uninstall trento-server
```
